@@ -20,6 +20,8 @@ The options are:
 * ``REDIS_HOST``: The Redis host adress to connect to
 * ``REDIS_PORT``: The Redis port to connect to
 * ``REDIS_DB``: The Redis database to connect to
+* ``REDIS_PASSWORD``: The password of the Redis database to connect to
+* ``REDIS_USERNAME``: The username of the Redis database to connect to
 """
 
 
@@ -28,6 +30,7 @@ import os
 from argparse import ArgumentParser
 from dotenv import load_dotenv
 from metrics import Byte, Time
+from pprint import pformat
 
 
 logger = logging.getLogger(__name__)
@@ -42,10 +45,17 @@ def strtobool(s):
     return s not in {False, "0", "false", "no"}
 
 
+def asdict():
+    return {
+        key: value for key, value in globals().items()
+        if key.isupper() and not key.startswith('__')
+    }
+
+
 def _setup():
     global HOST, PORT, MAXSIZE, IDENTSIZE
     global DEFAULT_LANGUAGE, DEFAULT_MAXUSAGE, DEFAULT_LIFETIME
-    global REDIS_HOST, REDIS_PORT, REDIS_DB
+    global REDIS_HOST, REDIS_PORT, REDIS_DB, REDIS_PASSWORD, REDIS_USERNAME
 
     cli = ArgumentParser()
     cli.add_argument('--rtdbin-port', metavar="PORT", type=int, help="builtin server port")
@@ -67,6 +77,8 @@ def _setup():
     REDIS_HOST = os.getenv('REDIS_HOST', 'localhost')
     REDIS_PORT = int(os.getenv('REDIS_PORT', 6379))
     REDIS_DB = int(os.getenv('REDIS_DB', 0))
+    REDIS_PASSWORD = os.getenv('REDIS_PASSWORD') or None
+    REDIS_USERNAME = os.getenv('REDIS_USERNAME') or None
 
     # Configure logging
     if "gunicorn" in server_software:
@@ -86,10 +98,6 @@ def _setup():
         "Detected WSGI server: %s.",
         server_software or 'builtin wsgiref (unsafe for production)'
     )
-    logger.debug(
-        "Runtime configuration:\n  %s", "\n  ".join([
-        f"{key}: {val}" for key, val in globals().items()
-        if key.isupper() and not key.startswith('__')
-    ]))
+    logger.debug("Runtime configuration:\n%s", pformat(asdict()))
 
 _setup()
