@@ -1,12 +1,18 @@
 import bottle
 import unittest
 import urllib.request as urlreq
-from urllib.error import HTTPError
-from bin.models import Snippet
+
 from bottle import template as bottle_template
 from html.parser import HTMLParser
 from threading import Thread
 from unittest.mock import patch, MagicMock
+from urllib.parse import urlencode
+from urllib.error import HTTPError
+
+import bin as binmod
+from bin.models import Snippet
+from .fake_config import FakeConfig
+from .html_sanitizer import HTMLSanitizer
 
 
 def make_snippet(ident, code):
@@ -16,90 +22,18 @@ snippet_lipsum = make_snippet('lipsum', code="Lorem ipsum dolor sit amet")
 snippet_python = make_snippet('egg', code='print("Hello world")')
 snippet_htmlxss = make_snippet('htmlxss', code='<script>alert("XSS");</script>')
 snippet_classified = make_snippet('classified', code='T_xJV3P^FcvYijzH')
+snippet_brainfuck = make_snippet('brainfuck', code='++++++++++[>+>+++>+++++++>++++++++++<<<<-]>>>++++.+++++++++.')
 
 UA_HUMAN = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:85.0) Gecko/20100101 Firefox/85.0"
 UA_BOT = "Mozilla/5.0 (compatible; Discordbot/2.0; +https://discordapp.com)"
 
 
-class HTMLSanitizer(HTMLParser):
-    def __init__(self, testcase):
-        self.testcase = testcase
-        self.pairtags = {
-            'a', 'abbr', 'acronym', 'address', 'applet', 'article', 'aside',
-            'audio', 'b', 'basefont', 'bdi', 'bdo', 'big', 'blockquote',
-            'body', 'button', 'canvas', 'caption', 'center', 'cite', 'code',
-            'colgroup', 'data', 'datalist', 'dd', 'del', 'details', 'dfn',
-            'dialog', 'dir', 'div', 'dl', 'doctype', 'dt', 'em', 'fieldset',
-            'figcaption', 'figure', 'font', 'footer', 'form', 'frame',
-            'frameset', 'head', 'header', 'hgroup', 'h1', 'h2', 'h3', 'h4',
-            'h5', 'h6', 'html', 'i', 'iframe', 'ins', 'kbd', 'label', 'legend',
-            'li', 'main', 'map', 'mark', 'menu', 'menuitem', 'meter', 'nav',
-            'noframes', 'noscript', 'object', 'ol', 'output', 'p', 'picture',
-            'pre', 'progress', 'q', 'rp', 'rt', 'ruby', 's', 'samp', 'script',
-            'section', 'select', 'small', 'span', 'strike', 'strong', 'style',
-            'sub', 'summary', 'sup', 'svg', 'table', 'tbody', 'td', 'template',
-            'textarea', 'tfoot', 'th', 'thead', 'time', 'title', 'tr', 'tt',
-            'u', 'ul', 'var', 'video',
-        }
-        self.singletags = {
-            'area', 'base', 'br', 'col', 'command', 'embed', 'hr', 'img',
-            'input', 'keygen', 'link', 'meta', 'param', 'source', 'track',
-            'wbr',
-        }
-        self.stack = []
-        self.insvg = False
-        super().__init__()
-
-    def handle_starttag(self, starttag, attrs):
-        if self.stack and self.stack[-1] == 'svg':
-            return
-
-        if starttag in ('option', 'optgroup'):
-            # <option> and <optgroup> can be single or pair, no need to verify
-            # append them the the stack, we just verify their immediate parent
-            # tag is a <select> or a <datalist>.
-            self.testcase.assertTrue(self.stack[-1] in ('select', 'datalist'),
-                "%s must be direct child of select or datalist")
-            return
-
-        self.testcase.assertIn(starttag, self.pairtags | self.singletags, "unknown html tag")
-        if starttag in self.pairtags:
-            self.stack.append(starttag)
-
-    def handle_startendtag(self, startendtag, attrs):
-        if self.stack and self.stack[-1] == 'svg':
-            return
-
-        if startendtag in ('option', 'optgroup'):
-            # <option> and <optgroup> can be single or pair, we just verify
-            # their immediate parent tag is a <select> or a <datalist>.
-            self.testcase.assertTrue(self.stack[-1] in ('select', 'datalist'),
-                "%s must be direct child of select or datalist")
-            return
-
-        self.testcase.assertIn(startendtag, self.singletags)
-
-    def handle_endtag(self, endtag):
-        if self.stack[-1] == 'svg' and endtag != 'svg':
-            return
-
-        if endtag in ('option', 'optgroup'):
-            # <option> and <optgroup> can be single or pair, they were not
-            # added to the stack, we just verify their immediate parent is
-            # a <select> or a <datalist>
-            self.testcase.assertTrue(self.stack[-1] in ('select', 'datalist'),
-                "%s must be direct child of select or datalist")
-            return
-
-        starttag = self.stack.pop()
-        self.testcase.assertEqual(starttag, endtag, "malformatted html")
-
-
 class TestController(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
         print("Starting builtin server in a daemon thread")
+
         th = Thread(target=bottle.run, daemon=True, kwargs={
             "host": "localhost",
             "port": 8012,
@@ -118,16 +52,21 @@ class TestController(unittest.TestCase):
             else:
                 break
 
-        bottle.template = MagicMock()
-        bottle.template.side_effect = bottle_template
+        cls.config = FakeConfig()
+        patch_config = patch.object(binmod.controller, 'config', cls.config)
+        patch_config.start()
+        cls.addClassCleanup(patch_config.stop)
 
-    @classmethod
-    def tearDownClass(cls):
-        bottle.template = bottle_template
+        mock_bt_tmpl = MagicMock()
+        mock_bt_tmpl.side_effect = bottle.template
+        patch_bt_tmpl = patch.object(bottle, 'template', mock_bt_tmpl)
+        patch_bt_tmpl.start()
+        cls.addClassCleanup(patch_bt_tmpl.stop)
 
     def setUp(self):
         self.html_sanitizer = HTMLSanitizer(self)
         bottle.template.reset_mock()
+        self.config.clear()
 
     # =====
 
@@ -154,6 +93,7 @@ class TestController(unittest.TestCase):
             MockSnippet.get_by_id.return_value = snippet_lipsum
             with urlreq.urlopen("http://localhost:8012/lipsum") as res:
                 bottle.template.assert_called_once()
+                MockSnippet.get_by_id.assert_called_with(snippet_lipsum.id)
                 self.assertEqual(res.status, 200)
                 self.html_sanitizer.feed(res.read().decode())
 
@@ -161,6 +101,7 @@ class TestController(unittest.TestCase):
             MockSnippet.get_by_id.return_value = snippet_python
             with urlreq.urlopen("http://localhost:8012/egg.py") as res:
                 bottle.template.assert_called_once()
+                MockSnippet.get_by_id.assert_called_with(snippet_python.id)
                 self.assertEqual(res.status, 200)
                 self.html_sanitizer.feed(res.read().decode())
 
@@ -169,15 +110,44 @@ class TestController(unittest.TestCase):
             MockSnippet.get_by_id.return_value = snippet_lipsum
             with urlreq.urlopen("http://localhost:8012/raw/lipsum") as res:
                 bottle.template.assert_not_called()
+                MockSnippet.get_by_id.assert_called_with(snippet_lipsum.id)
                 self.assertEqual(res.status, 200)
                 self.assertEqual(res.read().decode(), snippet_lipsum.code)
 
+
             MockSnippet.get_by_id.return_value = snippet_python
             with urlreq.urlopen("http://localhost:8012/raw/egg.py") as res:
                 bottle.template.assert_not_called()
+                MockSnippet.get_by_id.assert_called_with(snippet_python.id)
                 self.assertEqual(res.status, 200)
                 self.assertEqual(res.read().decode(), snippet_python.code)
 
+    def test_get_wrong_extension(self):
+        with patch('bin.models.Snippet') as MockSnippet, \
+             patch('bin.controller.highlight') as MockHighlight:  # from-imported
+            MockSnippet.get_by_id.return_value = snippet_lipsum
+            MockHighlight.return_value = ''
+
+            with urlreq.urlopen("http://localhost:8012/lipsum.idontexist") as res:
+                self.assertEqual(res.status, 200)
+
+            MockSnippet.get_by_id.assert_called_with(snippet_lipsum.id)
+            MockHighlight.assert_called_with(snippet_lipsum.code, 'text')
+
+    def test_get_hidden_extension(self):
+        with patch('bin.models.Snippet') as MockSnippet, \
+             patch('pygments.highlight') as MockHighlight:
+            MockSnippet.get_by_id.return_value = snippet_brainfuck
+            MockHighlight.return_value = ''
+
+            with urlreq.urlopen("http://localhost:8012/brainfuck.bf") as res:
+                self.assertEqual(res.status, 200)
+
+            MockSnippet.get_by_id.assert_called_with(snippet_brainfuck.id)
+            MockHighlight.assert_called_once()
+            self.assertEqual(MockHighlight.call_args[0][0], snippet_brainfuck.code)
+            self.assertIn('BrainfuckLexer', str(MockHighlight.call_args[0][1]))
+
     def test_against_xss(self):
         testcase = self
 
@@ -224,3 +194,27 @@ class TestController(unittest.TestCase):
             with urlreq.urlopen(req) as res:
                 MockSnippet.get_by_id.assert_called()
                 self.assertEqual(res.status, 200)
+
+    def test_new_form(self):
+        with patch('bin.models.Snippet') as MockSnippet:
+            MockSnippet.create.return_value = snippet_lipsum
+            MockSnippet.get_by_id.return_value = snippet_lipsum
+
+            data = urlencode({'code': snippet_lipsum.code}).encode()
+            req = urlreq.Request(
+                'http://localhost:8012/new',
+                data=data,
+                headers={
+                    'Content-Length': str(len(data))
+                }
+            )
+
+            with urlreq.urlopen(req) as res:
+                MockSnippet.create.assert_called_with(
+                    snippet_lipsum.code,
+                    self.config.DEFAULT_MAXUSAGE,
+                    self.config.DEFAULT_LIFETIME,
+                    '',  # parentid
+                    None,  # token
+                )
+                self.assertEqual(res.url, 'http://localhost:8012/lipsum.txt')
