@@ -8,9 +8,9 @@ class TestHighlight(unittest.TestCase):
         try:
             pygments.lexers.get_lexer_by_name(lang)
             return True
-        except pygments.utils.ClassNotFound:
+        except pygments.util.ClassNotFound:
             return False
 
     def test_languages_has_formatter(self):
-        for lang in highlight.languages:
-            self.assertTrue(self.has_language_formatter(lang[1]))
+        for _ext, lang in highlight.languages:
+            self.assertTrue(self.has_language_formatter(lang))
